/* ------------------------- REXX -------------------------- */
/* 95-07-10 or decode HTML files                             */
/* 96-01-15 or reworked                                      */
/* 96-04-17 or try to support <table>                        */
/* 96-06-29 or rework                                        */
/* --------------------------------------------------------- */
/* call:    htm2txt infile                                   */
/* output:  infile-name.TXT                                  */
/*                                                           */
/* recognised tags:                                          */
/*                                                           */
/*   all tags as supported by webex 1.1b                     */
/*                                                           */
/* special tags:                                             */
/*                                                           */
/*   <trace>                            trace '?i'           */
/*   <exit>                             exit immediately     */
/*                                                           */
/* recognised substitute variables see variable 'consts'     */
/*            tab-char '	' will be substituted by 10 blanks  */
/*                                                           */
/* notes:                                                    */
/*                                                           */
/*   all tags are converted as 'best fit'.                   */
/*   the image a browser produces will not be met.           */
/*                                                           */
/* --------------------------------------------------------- */
   linemax=80                /* maximum line length          */
   pixlbyt= 6                /* pixels per byte for tables   */
   editor ='e'               /* editor for output file       */

   parse upper arg ifiname .
   if ifiname='' then                              exit 4

   parse var ifiname fn '.' ext
   if ext='' then ifiname=ifiname'.HTM'
   ofiname = fn'.TXT'
/* --------------------------------------------------------- */
/* constants as known of today                               */
/*                                                           */
   consts=       "&space '20'x"
   consts=consts "&szlig   "
   consts=consts "&auml    "
   consts=consts "&Auml    "
   consts=consts "&ouml    "
   consts=consts "&Ouml    "
   consts=consts "&uuml    "
   consts=consts "&Uuml    "
   consts=consts "&#32   '20'x"
   consts=consts "&#58     :"
   consts=consts "&#60     <"
   consts=consts "&#62     >"
   consts=consts "&#91     ["
   consts=consts "&#93     ]"
   consts=consts "&#146    '"   /*              */
   consts=consts '&quot    "'
   consts=consts '&#145    "'   /*              */
   consts=consts '&#147    "'   /*              */
   consts=consts '&#148    "'   /*              */
   consts=consts "&#160  'a0'x"
   consts=consts "&#176    "   /*  EBCDIC !!!  */
   consts=consts "&#185    '"   /*              */
   consts=consts "&mdash   -"
   consts=consts "&lt      <"
   consts=consts "&gt      >"
   consts=consts "&amp   '00'x"
   consts=consts "&copy  '20'x"
/* --------------------------------------------------------- */
/* read infile                                               */
/*                                                           */
   nl ='0d'x                 /* new line character           */
   ifi=''
   say
   say 'reading' ifiname
   say
   call stream ifiname,'c','close'
   do i=1 while chars(ifiname)>1
     ifi = ifi||linein(ifiname)||nl
     end
   call stream ifiname,'c','close'
   say
   say i-1 'records read from' ifiname
   say
/* --------------------------------------------------------- */
/* format outfile lines                                      */
/*                                                           */
   ofi.0=0                   /* out file controls            */
   dlspaces  =''             /* <DL>-spaces                  */
   lispaces  =''             /* <LI>-spaces                  */
   indents   =0              /* number of indents            */
   blanklines=0              /* number of blank lines        */
   linelen   =linemax        /* max. linelength              */
   outtext   =''             /* initial text                 */

                             /* switches:                    */
   swi_pre   = 0             /* switch PRE                   */
   swi_tbl   = 0             /* switch table definition      */
   swi_cnt   = 0             /* switch center text           */
   swi_cat   = 0             /* switch concatenate           */
   swi_trc   = 0             /* switch trace                 */

   call out 'Extracted from' ifiname',' date()',' left(time(),5)
   call out

/* --------------------------------------------------------- */
/* scan input stream                                         */
/*                                                           */
   call charout ,'processing token '
   text=''

   do count=1 while length(ifi)>0

     call charout ,format(count,5) copies('08'x,6)

     if swi_trc then trace '?i'

                               /* check next line            */
     parse var ifi parttext '<' tag '>' ifi
                               /* process text               */
     select
       when swi_pre            then call process_preformatted
       when strip(parttext)=nl then nop
       otherwise               do
                                 do while pos(nl,parttext)>0
                                   parse var parttext a (nl) b
                                   parttext=strip(a) strip(b)
                                   end
                                 if swi_cat then text=text||parttext
                                 else do
                                   if text='' then text=     parttext
                                              else text=text parttext
                                   end
                                 end
       end
                               /* process tag                */
     tag=translate(tag,' ',nl)
     if left(tag,1)='!' then tag='!' substr(tag,2)
     parse var tag tag options
     tag=translate(tag)
     swi_cat=0

     select
       when tag='TRACE'          then  swi_trc=1
       when tag='EXIT'           then  signal finish

       when tag='!'              then  call out '***' options '***'
       when tag='FONT'           then  swi_cat=1
       when tag='UL',
       |    tag='OL',
       |    tag='DL',
       |    tag='DIR',
       |    tag='MENU',
                                 then do
                                       call out text
                                       call out ' '
                                       lispaces=' * '
                                       indents=indents+1
                                       end
       when tag='LI'             then  call out text
       when tag='DT'             then do
                                       call out text
                                       lispaces=' * '
                                       if indents>0 then indents=indents-1
                                       end
       when tag='DD'             then do
                                       call out text
                                       lispaces='    '
                                       indents=indents+1
                                       end
       when tag='/UL',
       |    tag='/OL',
       |    tag='/DL',
       |    tag='/DIR',
       |    tag='/MENU',
                                 then do
                                       call out text
                                       lispaces=''
                                       if indents>0 then indents=indents-1
                                       call out ' '
                                       end
       when tag='CENTER',
       |    tag='CENTRE',
                                 then  swi_cnt=1
       when tag='/CENTER',
       |    tag='/CENTRE',
                                 then do
                                       swi_cnt=0
                                       call out text
                                       end
       when tag='P',
       |    tag='/TITLE',
       |    tag='/CENTER',
       |    tag='/CENTRE',
                                 then  call out text
       when tag='/HEAD',
                                 then do
                                       call out text
                                       call out ' '
                                       end
       when tag='PRE'            then do
                                       swi_pre=1
                                       linelen=parmval('WIDTH=',options)
                                       end
       when tag='/PRE'           then do
                                       swi_pre=0
                                       linelen=linemax
                                       end

       when tag='HR'             then call out copies('-',linelen)

       when tag='H1',
       |    tag='H2',
       |    tag='H3',
       |    tag='H4',
       |    tag='/H1',
       |    tag='/H2',
       |    tag='/H3',
       |    tag='/H4',
                                 then do
                                       call out text
                                       call out ' '
                                       end

       when tag='TABLE'          then do
                                       call out text
                                       call out ' '
                                       swi_tbl=1
                                       end
       when tag='TR'             then do
                                       tbl_col=0
                                       tbl_max=0
                                       end
       when tag='TD'             then do
                                       z=parmval('COLSTART=',options)
                                       if z=0 then tbl_col=tbl_col+1
                                              else tbl_col=z
                                       if tbl_max<tbl_col then tbl_max=tbl_col
                                       p=parmval('WIDTH=',options)
                                       if p>0 then do
                                         select
                                           when right(p,3)='PIX' then do
                                             parse var p n 'PIX' .
                                             tbl_col.tbl_col=n%pixlbyt
                                             end
                                           when right(z,1)='%'   then do
                                             parse var p n '%' .
                                             tbl_col.tbl_col=(n*100)%linelen
                                             end
                                           otherwise tbl_col.tbl_col=p
                                           end
                                         end
                                       tbl_row.tbl_col=1
                                       end
       when tag='/TD'            then do
                                       if swi_tbl then call save_table_text
                                       end
       when tag='/TR'            then do
                                      if swi_tbl then do
                                         if datatype(tbl_col.1,'N') then do
                                           do i=1 to tbl_max
                                             tbl_col.i=linelen%tbl_max
                                             end
                                           end
                                         ymax=1
                                         do i=1 to tbl_max
                                           if ymax<tbl_row.i then ymax=tbl_row.i
                                           end
                                         do y=1 to ymax
                                           do i=1 to tbl_max
                                             if left(tbl_txt.i.y,7)='TBL_TXT',
                                                then tbl_txt.i.y=''
                                             if (strip(tbl_txt.i.y)='')&(i=1),
                                                then tbl_txt.i.y='_'
                                             text=text left(tbl_txt.i.y,tbl_col.i)
                                             tbl_txt.i.y=''
                                             end
                                           text=strip(text)
                                           if text='_' then call o text
                                           text=''
                                           end
                                         end
                                       end
       when tag='/TABLE'         then do
                                       blanklines=0
                                       call out ' '
                                       swi_tbl=0
                                       end
       when tag='BR'             then do
                                       if swi_tbl then call save_table_text
                                       else call out text
                                       end
       otherwise nop
       end
                               /* all finished               */
     end
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
   signal sub1_end

   save_table_text:

     if strip(text)=''         then return

     y=tbl_row.tbl_col
     if datatype(tbl_col.y,'N') then l=linelen%tbl_max
                                 else l=tbl_col.tbl_col
     text=subs(text)
     do while length(text)>l
       z=lastpos(' ',text,l)
       if z=0 then leave
       tbl_txt.tbl_col.y=strip(left(text,z))
       tbl_row.tbl_col=tbl_row.tbl_col+1
       y=tbl_row.tbl_col
       text=substr(text,z+1)
       end
     tbl_txt.tbl_col.y=strip(text)
     tbl_row.tbl_col=tbl_row.tbl_col+1
     text=''
                                                      return
   sub1_end:
/* --------------------------------------------------------- */
/* write outfile                                             */
/*                                                           */
finish:

   say
   '@erase' ofiname '2>NUL'
   do i=1 to ofi.0
     call lineout ofiname,ofi.i
     end
   call lineout ofiname

   'start /f' editor ofiname /*  <=== edit result            */
/* --------------------------------------------------------- */
                                                      exit 0
/* ========================================================= */
/* --------------------------------------------------------- */
/* process preformatted                                      */
/*                                                           */
process_preformatted:

   do while length(parttext)>0
     parse var parttext outtext (nl) parttext
     call out outtext
     end
                                                    return
/* --------------------------------------------------------- */
/* extract parameter values                                  */
/*                                                           */
parmval: procedure; parse upper arg key,string

   z=pos(key,string)
   if z=0 then                                  return 0
   string=substr(string,z)
   parse var string . '"' val '"' .
   val =translate(strip(val))
                                                return val
/* --------------------------------------------------------- */
/* do output lines                                           */
/*                                                           */
out:

   oli=subs(strip(arg(1)))
   oll=length(oli)
                     /* do not output more than 1 blank line */
   if oll=0 then do
     if blanklines>0 then                       return
     blanklines=blanklines+1
     end

   if linelen>0 then do
     do while oll>linelen
       z=lastpos(' ',oli,linelen)
       if z=0 then z=oll
       if (z>0) then do
                     call o left(oli,z)
                     oli=strip(substr(oli,z+1))
                     oll=length(oli)
                     end
       end
     end
   call o oli
   if oll>0 then blanklines=0
   text=''
                                                     return
o: procedure expose swi_cnt linelen indents dlspaces lispaces ofi.
                            parse arg ooo
   if swi_cnt then do
     z=(linelen-length(ooo))%2
     if z>0 then prefix=copies(' ',z)
            else prefix=''
     end
   else do
     prefix=copies(' ',indents)||lispaces||dlspaces
     end
   ofi.0=ofi.0+1; z=ofi.0; ofi.z=prefix||ooo
                                                     return
/* --------------------------------------------------------- */
/* substitute constants                                      */
/*                                                           */
   subs: procedure expose consts;

     l = arg(1)
                               /* check for tab chars        */
     t='09'x
     s=copies(' ',10)
     z=pos(t,l)
     do while z>0
       l=left(l,z-1)||s||substr(l,z+1)
       z=pos(t,l)
       end
                               /* check for variables        */
     z=pos('&',l)
     if z=0 then                              return l

     do while z > 0
       head  = left(l,z-1)
       token = substr(l,z)
       do i=1 to words(consts) by 2
         a=word(consts,i)
         b=length(a)
         c=left(token,b)
         d=word(consts,i+1)
         if right(d,2)="'x" then interpret "d="d
         if c=a then do
           head=head||d
           token=substr(token,b+2)
           leave
           end
         end
       if i>words(consts) then do
         token='?'substr(token,2)
         end
       l = head||token
       z=pos('&',l)
       end
                               return translate(l,'&','00'x)
/* --------------------------------------------------------- */
